# Руководство по участию в проекте LiteLLM GigaChat Integration

Спасибо за интерес к проекту! Мы приветствуем любые формы участия - от сообщений об ошибках до предложений новых функций и pull request'ов.

## 🚀 Как начать

### Быстрый старт для разработчиков

1. **Форкните репозиторий** на GitHub
2. **Клонируйте ваш форк**:
   ```bash
   git clone https://github.com/ваш-username/litellm-gigachat.git
   cd litellm-gigachat
   ```
3. **Создайте виртуальное окружение**:
   ```bash
   python -m venv venv
   source venv/bin/activate  # Linux/Mac
   # или
   venv\Scripts\activate  # Windows
   ```
4. **Установите зависимости**:
   ```bash
   pip install -r requirements.txt
   pip install -r requirements-dev.txt  # если есть dev зависимости
   ```
5. **Настройте переменные окружения**:
   ```bash
   cp .env.example .env
   # Добавьте ваш GIGACHAT_AUTH_KEY
   ```

## 📋 Типы участия

### 🐛 Сообщения об ошибках

Перед созданием issue проверьте:
- [ ] Существующие [Issues](https://github.com/stp008/litellm-gigachat/issues)
- [ ] [FAQ.md](FAQ.md) - возможно, решение уже есть
- [ ] Последнюю версию проекта

**Шаблон для сообщения об ошибке:**
```markdown
## Описание ошибки
Краткое описание проблемы.

## Шаги для воспроизведения
1. Запустите команду `...`
2. Откройте файл `...`
3. Увидите ошибку

## Ожидаемое поведение
Что должно было произойти.

## Фактическое поведение
Что произошло на самом деле.

## Окружение
- OS: [например, macOS 14.0]
- Python: [например, 3.9.7]
- LiteLLM: [например, 1.65.1]
- Версия проекта: [например, 1.1.0]

## Логи
```
Вставьте логи ошибок здесь
```

## Дополнительная информация
Любая дополнительная информация, скриншоты и т.д.
```

### 💡 Предложения улучшений

**Шаблон для предложения функции:**
```markdown
## Описание функции
Краткое описание предлагаемой функции.

## Мотивация
Почему эта функция нужна? Какую проблему она решает?

## Детальное описание
Подробное описание того, как должна работать функция.

## Возможная реализация
Если у вас есть идеи по реализации, опишите их.

## Альтернативы
Рассматривали ли вы альтернативные решения?
```

### 🔧 Pull Requests

#### Процесс создания PR

1. **Создайте ветку** для вашей функции:
   ```bash
   git checkout -b feature/название-функции
   # или
   git checkout -b fix/описание-исправления
   ```

2. **Внесите изменения** и зафиксируйте их:
   ```bash
   git add .
   git commit -m "feat: добавить поддержку новой модели GigaChat"
   ```

3. **Запустите тесты**:
   ```bash
   python -m pytest tests/
   python test_basic_functionality.py
   python test_cline_integration.py
   ```

4. **Отправьте изменения**:
   ```bash
   git push origin feature/название-функции
   ```

5. **Создайте Pull Request** на GitHub

#### Требования к PR

- [ ] **Описание**: Четкое описание изменений
- [ ] **Тесты**: Новые функции должны быть покрыты тестами
- [ ] **Документация**: Обновите документацию при необходимости
- [ ] **Стиль кода**: Следуйте существующему стилю
- [ ] **Коммиты**: Используйте [Conventional Commits](#conventional-commits)

## 📝 Стандарты разработки

### Стиль кода

**Python код:**
- Следуйте [PEP 8](https://pep8.org/)
- Используйте type hints где возможно
- Максимальная длина строки: 88 символов (Black formatter)
- Используйте docstrings для функций и классов

**Пример хорошего кода:**
```python
def get_gigachat_token(force_refresh: bool = False) -> str:
    """
    Получить действующий токен GigaChat.
    
    Args:
        force_refresh: Принудительно обновить токен
        
    Returns:
        Действующий токен доступа
        
    Raises:
        AuthenticationError: Если не удалось получить токен
    """
    manager = get_global_token_manager()
    return manager.get_token(force_refresh=force_refresh)
```

### Тестирование

**Структура тестов:**
```
tests/
├── test_basic_functionality.py    # Основные функции
├── test_token_manager.py          # Управление токенами
├── test_content_handler.py        # Обработка контента
├── test_cline_integration.py      # Интеграция с Cline
└── test_proxy.py                  # Прокси-сервер
```

**Запуск тестов:**
```bash
# Все тесты
python -m pytest tests/ -v

# Конкретный тест
python test_basic_functionality.py

# С покрытием кода
python -m pytest tests/ --cov=src --cov-report=html
```

**Написание тестов:**

```python
import pytest
from src.litellm_gigachat.core import TokenManager


def test_token_manager_initialization():
    """Тест инициализации TokenManager."""
    manager = TokenManager()
    assert manager is not None
    assert not manager.has_token()


def test_token_refresh():
    """Тест обновления токена."""
    manager = TokenManager()
    # Мокаем API вызов
    with patch('requests.post') as mock_post:
        mock_post.return_value.json.return_value = {
            'access_token': 'test_token',
            'expires_at': int(time.time()) + 1800
        }

        token = manager.get_token(force_refresh=True)
        assert token == 'test_token'
```

### Документация

**Обновление документации:**
- Обновите `README.md` для новых функций
- Добавьте примеры в `examples/`
- Обновите `FAQ.md` для частых вопросов
- Добавьте запись в `CHANGELOG.md`

**Комментарии в коде:**
```python
# Хорошо: объясняет ПОЧЕМУ
# Преобразуем массив content в строку для совместимости с GigaChat
content_str = flatten_content_array(message.content)

# Плохо: объясняет ЧТО (и так видно из кода)
# Присваиваем переменной значение
content_str = flatten_content_array(message.content)
```

## 🏷️ Conventional Commits

Используйте [Conventional Commits](https://www.conventionalcommits.org/ru/v1.0.0/) для сообщений коммитов:

**Формат:**
```
<тип>[область]: <описание>

[тело коммита]

[подвал]
```

**Типы коммитов:**
- `feat`: новая функция
- `fix`: исправление ошибки
- `docs`: изменения в документации
- `style`: форматирование, отсутствующие точки с запятой и т.д.
- `refactor`: рефакторинг кода
- `test`: добавление тестов
- `chore`: обновление задач сборки, настроек и т.д.

**Примеры:**
```bash
feat(token): добавить автоматическое обновление токенов
fix(cline): исправить ошибку "invalid JSON syntax"
docs(readme): обновить инструкции по установке
test(proxy): добавить тесты для прокси-сервера
refactor(core): улучшить структуру TokenManager
```

## 🔍 Области для участия

### 🚀 Приоритетные задачи

1. **Производительность**
   - Оптимизация обработки запросов
   - Кэширование токенов
   - Асинхронная обработка

2. **Интеграции**
   - Поддержка новых AI-ассистентов
   - Интеграция с популярными IDE
   - Веб-интерфейс для мониторинга

3. **Тестирование**
   - Увеличение покрытия тестами
   - Интеграционные тесты
   - Нагрузочное тестирование

4. **Документация**
   - Видео-туториалы
   - Переводы на другие языки
   - Примеры использования

### 🛠️ Технические улучшения

- **Docker поддержка**: Создание Dockerfile и docker-compose
- **CI/CD**: Настройка GitHub Actions
- **Мониторинг**: Интеграция с Prometheus/Grafana
- **Безопасность**: Аудит безопасности и исправления

### 📚 Документация и контент

- **Блог-посты**: Статьи о использовании
- **Примеры**: Реальные кейсы использования
- **Переводы**: Документация на английском языке
- **Видео**: Обучающие материалы

## 🎯 Roadmap проекта

### Версия 1.3.0 - Производительность
- [ ] Кэширование токенов
- [ ] Оптимизация памяти
- [ ] Метрики производительности
- [ ] Асинхронная обработка

### Версия 1.4.0 - Интеграции
- [ ] Поддержка Continue.dev
- [ ] Интеграция с Cursor
- [ ] Веб-интерфейс
- [ ] REST API для управления

### Версия 2.0.0 - Архитектура
- [ ] Микросервисная архитектура
- [ ] Docker контейнеры
- [ ] Kubernetes поддержка
- [ ] Облачное развертывание

## 🏆 Признание участников

Все участники проекта будут упомянуты в:
- `CONTRIBUTORS.md` файле
- Release notes
- README.md (для значительных вкладов)

### Типы участия

- 🐛 **Bug reports** - сообщения об ошибках
- 💡 **Ideas** - предложения улучшений
- 🔧 **Code** - программный код
- 📖 **Documentation** - документация
- 🎨 **Design** - дизайн и UX
- 💬 **Community** - помощь сообществу
- 🌍 **Translation** - переводы

## 📞 Связь с сообществом

### GitHub
- **Issues**: Для багов и предложений
- **Discussions**: Для общих вопросов
- **Pull Requests**: Для изменений кода

### Каналы связи
- **Email**: [ваш-email] для приватных вопросов
- **Telegram**: [ссылка на канал] для быстрой связи

## ❓ Часто задаваемые вопросы

### Как начать участвовать, если я новичок?

1. Начните с изучения [Issues с меткой "good first issue"](https://github.com/stp008/litellm-gigachat/labels/good%20first%20issue)
2. Прочитайте документацию и попробуйте запустить проект
3. Найдите опечатки в документации - это отличный первый вклад
4. Задавайте вопросы в Discussions

### Сколько времени занимает review PR?

Обычно 2-7 дней, в зависимости от сложности изменений. Мы стараемся отвечать как можно быстрее.

### Можно ли участвовать без знания Python?

Да! Вы можете помочь с:
- Документацией
- Тестированием
- Переводами
- Дизайном
- Сообщениями об ошибках

### Как предложить крупное изменение?

1. Создайте Issue с описанием предложения
2. Обсудите с мейнтейнерами
3. Создайте RFC (Request for Comments) если нужно
4. Начните реализацию после одобрения

---

**Спасибо за участие в проекте LiteLLM GigaChat Integration!** 🚀

Ваш вклад помогает сделать интеграцию российских AI-технологий более доступной для разработчиков.
